==> zipper-1.0.tar.gz: zipper/dccchat.tcl <==
package provide dccchat 1.0

proc init_dcc_connection {tnick dccbaseport} {
	global longip
	global srvcPort 
	global s_sock
	
	set s_sock [socket -server "connect_dcc_connection $tnick" $dccbaseport ]

}
proc connect_dcc_connection {tnick sock addr port } {
	global s_sock

	fileevent "$sock" readable [list read_dcc_sock "$sock"]
	fconfigure "$sock" -buffering line
	winmannew "dcc$sock" "DCC:$tnick" "dcc" "0 $tnick $sock"
	set response [get_dcc_nick_from_socket $sock]
	insert_chan_list "DCC:$response"
	set_chan_activity 1 "dcc$sock"
	send_to_window "dcc$sock" "Talking to $tnick...\n"
	close $s_sock
}

proc accept_dcc_connection {tnick dccport} {
# open the connection to the DCC server...
	global xip

	set sock [socket $xip $dccport]
	winmannew "dcc$sock" "DCC:$tnick" "dcc" "1 $tnick $sock"
	set response [get_dcc_nick_from_socket $sock]
	send_to_window "dcc$sock"  "Talking to $tnick...\n"
	insert_chan_list "DCC:$response"
	set_chan_activity 1 "dcc$sock"
	fileevent $sock readable [list read_dcc_sock "$sock"] 
	fconfigure $sock  -buffering line  

}

proc read_dcc_sock {sock} {

  set l [gets $sock]
  if {[eof $sock]} {
	close $sock
	send_to_window "dcc$sock" "DCC Session Closed\n"
	.zipperdcc$sock.entry configure -state disabled
        set eventLoop "done"
  } else {
    handle_dcc_message  $sock $l
  }
}     

proc read_dcc_user_in {sock}  {
	global nick
	global esvrSock
	global history
	global historyidx


	set l [.zipperdcc$sock.entry get]
	.zipperdcc$sock.entry delete 0 end 

	set hname [string tolower [get_dcc_nick_from_socket $sock] ]
	set historyidx(dcc:$hname) 30 
 	lappend history(dcc:$hname) $l    
	roll_off_hist dcc:$hname


#default is to send to current dcc window

  if ![string compare "/" [string index $l 0] ] {
        set l [string trimleft $l "/"]
        set_command $esvrSock
  } else {
          send_to_window "dcc$sock" "> $l\n"
	  send_to_dcc_server $sock "$l"
  }    

}


proc get_dcc_nick_from_socket sock {
	global chaninfo

	set index "dcc$sock"
	set tnick $chaninfo($index,nickchan)
	return $tnick

}
proc get_dcc_sock_from_nick nick {
	global chaninfo
	
	set list [array names chaninfo "dcc*,nickchan" ]
	foreach index $list {
		set x $chaninfo($index)
		if ![string compare $nick $x] {
			set y [lindex [split $index ","] 0]
			set tsock $chaninfo($y,socket)
			break
		}
	}
	
	return $tsock
}

proc remove_dcc_from_array {sock} {

	global chaninfo
	global chanrev
	set index "dcc$sock"

	unset chaninfo($index,name)
	unset chaninfo($index,nickchan)
	unset chaninfo($index,type)
	unset chaninfo($index,stat)
	unset chaninfo($index,history)
	unset chaninfo($index,historyidx)
	unset chaninfo($index,winnumber)
	unset chaninfo($index,socket)
	unset chanrev(.zipperdcc$sock)

}
proc handle_dcc_message {sock l} {

	set tnick [get_dcc_nick_from_socket $sock]

	if ![string compare "\001" [string index $l 0] ] {
		set l [string trim $l "\001" ]
		set l [string trimleft $l "ACTION" ]
		set l [string trimleft $l " " ]
		send_to_window "dcc$sock" "* $tnick $l\n"
	} else {	
		send_to_window "dcc$sock" "<$tnick> $l\n"
	}

}

proc new_dcc_window {w name sock} {

global response
global textfg
global textbg
global mirccolors
global actwin

set w ".zipperdcc$sock"
toplevel $w
wm iconname $w "$name"
#set font  -adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*
set font 8x13

wm title $w $name
wm iconbitmap $w @zipperwinicon.bmp
set start 0.0
set length 0             

text $w.text  -state disabled -font $font -relief sunken -bd 2 -yscrollcommand "$w.scroll set" -setgrid 1 -height 20 -width 50  -foreground $mirccolors($textfg) -background $mirccolors($textbg)

scrollbar $w.scroll -command "$w.text yview"
entry $w.entry -bd 2 -relief sunken -width 30
pack $w.scroll -side right -fill y
pack $w.text -expand yes -fill both
pack $w.entry -expand no -fill both

wm iconify $w
focus $w.entry

bind $w.entry <KeyPress-Up>     {select_history [focus] -1 }
bind $w.entry <KeyPress-Down>   {select_history [focus] 1 }   
bind $w.entry <Control-KeyPress-b> {[focus] insert end "\002"}
bind $w.entry <Control-KeyPress-k> {[focus] insert end "\003"}
bind $w.entry <Control-KeyPress-i> {[focus] insert end "\026"}
bind $w.entry <Control-KeyPress-u> {[focus] insert end "\037"}
bind $w.entry <Control-KeyPress-g> {[focus] insert end "\007"}      
bind $w.entry <Destroy> {canceldcc $actwin  }
bind $w.entry <KeyPress-Return> "read_dcc_user_in $sock"
bind $w  <FocusIn> {update_chan_activity 0 [focus] }   

}                   



proc canceldcc {w} {

	set sockname [lindex [split [string range $w 10 end] "."] 0]
	closedcc $sockname
}

proc closedcc sock {

	set response [ get_dcc_nick_from_socket $sock]
	remove_dcc_from_array $sock
	remove_chan_list "DCC:$response"
	destroy ".zipperdcc$sock"
	close $sock

}

proc send_to_dcc_server {wsock l} {
  puts $wsock $l           ;# send the data to the server

} 

proc encode_ip {} {
    global longip
    set longip [exec encodeip $longip]
}
proc decode_ip {} {
	global longip
	global xip
		
	set xip [exec decodeip $longip]

}
proc dccchat {type tnick connectport dccport} {

global longip
global xip

if ![string compare "get" $type] {
	decode_ip
	accept_dcc_connection $tnick $connectport
} else {
	decode_ip
	init_dcc_connection $tnick $dccport
}
}

==> zipper-1.631.tar.gz: zipper/dccchat.tcl <==
package provide dccchat 1.63
### Control-Shift-B courtesy of Tim Howe


proc init_dcc_connection {tnick dccbaseport} {
	global longip
	global srvcPort 
	global s_sock
	
	set s_sock [socket -server "connect_dcc_connection $tnick" $dccbaseport ]

}
proc connect_dcc_connection {tnick sock addr port } {
	global s_sock

	fileevent "$sock" readable [list read_dcc_sock "$sock"]
	fconfigure "$sock" -buffering line
	winmannew "dcc$sock" "DCC:$tnick" "dcc" "0 $tnick $sock"
	set response [get_dcc_nick_from_socket $sock]
	insert_chan_list "DCC:$response"
	set_chan_activity 1 "dcc$sock"
	send_to_window "dcc$sock" "Talking to $tnick...\n"
	close $s_sock
}

proc accept_dcc_connection {tnick dccport} {
# open the connection to the DCC server...
	global xip

	set sock [socket $xip $dccport]
	winmannew "dcc$sock" "DCC:$tnick" "dcc" "1 $tnick $sock"
	set response [get_dcc_nick_from_socket $sock]
	insert_chan_list "DCC:$response"
	set_chan_activity 1 "dcc$sock"
	send_to_window "dcc$sock"  "Talking to $tnick...\n"
	fileevent $sock readable [list read_dcc_sock "$sock"] 
	fconfigure $sock  -buffering line  

}

proc read_dcc_sock {sock} {

  set l [gets $sock]
  if {[eof $sock]} {
	close $sock
	send_to_window "dcc$sock" "DCC Session Closed\n"
	.zipperdcc$sock.entry configure -state disabled
        set eventLoop "done"
  } else {
    handle_dcc_message  $sock $l
  }
}     

proc read_dcc_user_in {sock}  {
	global nick
	global esvrSock
	global history
	global historyidx


	set l [.zipperdcc$sock.entry get]
	.zipperdcc$sock.entry delete 0 end 

	set hname [string tolower [get_dcc_nick_from_socket $sock] ]
	set historyidx(dcc:$hname) 30 
 	lappend history(dcc:$hname) $l    
	roll_off_hist dcc:$hname


#default is to send to current dcc window

  if ![string compare "/" [string index $l 0] ] {
        set l [string trimleft $l "/"]
        set_command $l $esvrSock
  } else {
          send_to_window "dcc$sock" "> $l\n"
	  send_to_dcc_server $sock "$l"
  }    

}


proc get_dcc_nick_from_socket sock {
	global chaninfo

	set index "dcc$sock"
	set tnick $chaninfo($index,nickchan)
	return $tnick

}
proc get_dcc_sock_from_nick nick {
	global chaninfo
	
	set list [array names chaninfo "dcc*,nickchan" ]
	foreach index $list {
		set x $chaninfo($index)
		if ![string compare $nick $x] {
			set y [lindex [split $index ","] 0]
			set tsock $chaninfo($y,socket)
			break
		}
	}
	
	return $tsock
}

proc remove_dcc_from_array {sock} {

	global chaninfo
	global chanrev
	set index "dcc$sock"

	unset chaninfo($index,name)
	unset chaninfo($index,nickchan)
	unset chaninfo($index,type)
	unset chaninfo($index,stat)
	unset chaninfo($index,history)
	unset chaninfo($index,historyidx)
	unset chaninfo($index,winnumber)
	unset chaninfo($index,socket)
	unset chaninfo($index,logging)
	unset chaninfo($index,logfile)
	unset chaninfo($index,filedesc)
	unset chanrev(.zipperdcc$sock)

}
proc handle_dcc_message {sock l} {

	set tnick [get_dcc_nick_from_socket $sock]

	if ![string compare "\001" [string index $l 0] ] {
		set l [string trim $l "\001" ]
		set l [string trimleft $l "ACTION" ]
		set l [string trimleft $l " " ]
		send_to_window "dcc$sock" "* $tnick $l\n"
	} else {	
		send_to_window "dcc$sock" "<$tnick> $l\n"
	}

}

proc new_dcc_window {w name sock} {

global response
global textfg
global textbg
global mirccolors
global actwin
global installpath
global queryfont
global chaninfo
global chanrev

set w ".zipperdcc$sock"
toplevel $w
wm iconname $w "$name"
#set font  -adobe-courier-medium-r-normal--*-120-*-*-*-*-*-*
set font $queryfont

frame $w.labels
pack $w.labels -side top -fill x 

set idx [string tolower $chanrev($w) ]
checkbutton $w.labels.log -variable chaninfo($idx,logging) -text "Log" -command "toggle_logging $w"
pack $w.labels.log -side left

button $w.labels.inscol -text "Ins Color" -command { set c [pick_color]; $actwin insert end "\003$c" }
pack   $w.labels.inscol -side left  

wm title $w $name
wm iconbitmap $w @$installpath/zipperwinicon.xbm
set start 0.0
set length 0             

text $w.text  -state disabled -font $font -relief sunken -bd 2 -yscrollcommand "$w.scroll set" -setgrid 1 -height 20 -width 50  -foreground $mirccolors($textfg) -background $mirccolors($textbg)

scrollbar $w.scroll -command "$w.text yview"
entry $w.entry -bd 2 -relief sunken -width 30
pack $w.entry -side bottom -expand no -fill both
pack $w.scroll -side right -fill y
pack $w.text -expand yes -fill both

#wm iconify $w
center $w -50 -50
wm deiconify $w
focus $w.entry
bind $w <Unmap> "wm withdraw $w"
bind $w.entry <KeyPress-Up>     {select_history [focus] -1 }
bind $w.entry <KeyPress-Down>   {select_history [focus] 1 }   
bind $w.entry <Control-KeyPress-B> {[focus] insert end "\002"}
bind $w.entry <Control-KeyPress-k> {[focus] insert end "\003"}
bind $w.entry <Control-KeyPress-i> {[focus] insert end "\026"}
bind $w.entry <Control-KeyPress-u> {[focus] insert end "\037"}
bind $w.entry <Control-KeyPress-g> {[focus] insert end "\007"}      
bind $w.entry <KeyPress-Escape> { minimize [focus] }      
bind $w.entry <Destroy> {canceldcc $actwin  }
bind $w.entry <KeyPress-Return> "read_dcc_user_in $sock"
bind $w  <FocusIn> {update_chan_activity 0 [focus] }   

}                   



proc canceldcc {w} {

	set sockname [lindex [split [string range $w 10 end] "."] 0]
	closedcc $sockname
}

proc closedcc sock {

	set response [ get_dcc_nick_from_socket $sock]
	remove_chan_list "DCC:$response"
	closelog "dcc$sock"
	remove_dcc_from_array $sock
	destroy ".zipperdcc$sock"
	catch { close $sock }

}

proc send_to_dcc_server {wsock l} {
  puts $wsock $l           ;# send the data to the server

} 

proc encode_ip {} {
    global longip
    set longip [encodeip $longip]
}
proc decode_ip {} {
	global longip
	global xip
		
	set xip [decodeip $longip]

}
proc dccchat {type tnick connectport dccport} {

global longip
global xip

if ![string compare "get" $type] {
	decode_ip
	accept_dcc_connection $tnick $connectport
} else {
	decode_ip
	init_dcc_connection $tnick $dccport
}
}
